#  Kind of based on a Makefile (c) OPTIMUM May 1998      #
#  http://optimum.citeweb.net/index.html                 #

PROG	= LASERFIRE

CC	= gcc
CFLAGS	= -Wall -O2 -nostartfiles \
 -m486 -mno-fp-ret-in-387 -fforce-mem -fstrength-reduce \
 -fforce-addr -finline-functions -fcaller-saves -ffast-math
# --- Nicht implementiert fuer i386 -fschedule-insns -fschedule-insns2 ---
# -ffast-math - violate IEEE/ANSI but be faster, eg by assuming the arg of
#   sqrt is never negative
# -fstrength-reduce - try to eliminate iteration variables
# -fscheduleinsns[2] - reorder instructions [using 2nd pass]
# -fforce-mem, -fforce-addr: strange voodoo, see man gcc.
# -fcaller-saves: make more regs usable by doing more saves on stack
# -S -g3
# -dp annotate asm (.s) to tell why what was compiled which way 
# -02 optimize, but do not unroll or inline functions
# -Wall all warnings -pg profileinfo -s strip
# mit nostartfiles muss MAIN=_start statt MAIN=main sein 
# -S stop after generating ASM (-fverbose-asm does not seem to be useful)
# -g3 debugging info should even tell about local vars

DEF	= -D PROG=\"$(PROG)\" -D MAIN=_start -D SCALE=1 \
 -D ohneGAG=jupp -D wiedernichtUSEDSP -D auchnichtDEBUG

# SCALE ist eine Ganzzahl zwischen 1 und z.B. 4
# GAG definieren -> dreifache Abbildung desselben XImages in 1 Fenster...
# DSP neu: falls defined, Feuerhitze von stdin geregelt (fuer </dev/dsp)
# ... ist dann die Zahl der pro Bild analysierten Samplebytes ...
# DSPTEST: falls defined, Zahl der pro Testbildupdate analysierten stdin-bytes
# ... fuer Testbild als v/u Meter (lin, nicht log...)
# DEBUG: logo... if defined...

LIB	= -L/usr/lib/ -lm -L/usr/X11R6/lib/ -lX11
# wegen X11 Library ist stdlib leider unumgaenglich, -nostdlib geht nicht.

RM	= rm -f

OBJ	= grafik.o fire.o main.o fps.o testmuster.o laser.o stars.o \
		apfelmann.o plasma.o plasma_two.o oss.o musik.o inter.o

$(PROG):	$(OBJ)
		$(CC) $(CFLAGS) $(LIB) $(OBJ) -o $(PROG)
# -s bewirkt strip ... TAB, nicht Space hier im Makefile verwenden!

grafik.o:	grafik.c grafik.h
		$(CC) $(DEF) $(CFLAGS) -funroll-loops -c grafik.c -o grafik.o

main.o:		main.c main.h grafik.h typen.h fps.h fire.h laser.h starfield_3d.h
		$(CC) $(DEF) $(CFLAGS) -c main.c -o main.o

fps.o:		fps.c fps.h grafik.h
		$(CC) $(DEF) $(CFLAGS) -c fps.c -o fps.o

fire.o:		fire.c fire.h typen.h grafik.h
		$(CC) $(DEF) $(CFLAGS) -funroll-loops -c fire.c -o fire.o

apfelmann.o:	apfelmann.c apfelmann.h typen.h grafik.h
		$(CC) $(DEF) $(CFLAGS) -c apfelmann.c -o apfelmann.o

plasma.o:	plasma.c plasma.h typen.h grafik.h
		$(CC) $(DEF) $(CFLAGS) -c plasma.c -o plasma.o

plasma_two.o:	plasma_two.c plasma.h typen.h grafik.h
		$(CC) $(DEF) $(CFLAGS) -c plasma_two.c -o plasma_two.o

testmuster.o:	testmuster.c testmuster.h typen.h grafik.h
		$(CC) $(DEF) $(CFLAGS) -funroll-loops -c testmuster.c -o testmuster.o

laser.o:	laser.c laser.h typen.h grafik.h
		$(CC) $(DEF) $(CFLAGS) -c laser.c -o laser.o

stars.o:	starfield_3d.c starfield_3d.h typen.h grafik.h
		$(CC) $(DEF) $(CFLAGS) -c starfield_3d.c -o stars.o

oss.o:		oss.c oss.h typen.h grafik.h
		$(CC) $(DEF) $(CFLAGS) -c oss.c -o oss.o

musik.o:	musik.c musik.h typen.h grafik.h
		$(CC) $(DEF) $(CFLAGS) -c musik.c -o musik.o

inter.o:	interferenz.c interferenz.h typen.h grafik.h
		$(CC) $(DEF) $(CFLAGS) -c interferenz.c -o inter.o

clean:
		$(RM) $(OBJ)
		$(RM) core
		$(RM) .*~ *~
		# # # $(PROG)
