#include "apfelmann.h"

/* Pascal, Modus fuer MCGA von Eric  -  und 3/2000 auf X11 + C portiert  */
/* Animiert seit 9/99, Urversion 1995 */
/* "Solid guessing" neu 3/00 - Idee entfernt nach algorithm.txt von XaoS */

#ifdef APF_ITER
#define APF_FARBEN APF_ITER
#else
#define APF_FARBEN 47
#endif
// APF_FARBEN min 10, ansonsten je nach verfuegbarer Palette, z.B. 10, 24, 47

#define A_FP double
#include <values.h>
#define MIN_A_FP MINDOUBLE
// MINDOUBLE fuer i386 ist 2.22507385850720140e-308
// MINFLOAT fuer i386 ist ((float)1.17549435082228750e-38)
// keine Ahnung, ob die schon denormalisiert sind...

#ifdef BIG_APF_PIXELS
#define GFXr( x , y , z ) GFX( (x)<<1     , (y)<<1     , z )
#define GFXw( x , y , z ) GFX( (x)<<1     , (y)<<1     , z ) = \
                          GFX( ((x)<<1)+1 , (y)<<1     , z ) = \
                          GFX( (x)<<1     , ((y)<<1)+1 , z ) = \
                          GFX( ((x)<<1)+1 , ((y)<<1)+1 , z )
#else
#define GFXr GFX
#define GFXw GFX
#endif

int punkt_finden(const A_FP Re0, const A_FP Im0)
{
  A_FP ReVar, ImVar, ReTmp, ImTmp;
  int count;
  ReVar = ImVar = 0.0;
  ReTmp = ImTmp = 0.0;
  count = -1;
  do
  { 
    ImVar *= (ReVar + ReVar); ImVar += Im0;
    ReVar = ReTmp - ImTmp + Re0;
    count++;
    ReTmp = (ReVar * ReVar);
    ImTmp = (ImVar * ImVar);
  } while (((ReTmp + ImTmp) < 4.0) && (count < APF_FARBEN));
  // Komplexe Mandelbrot(2) Formel: x = x^2 + x0 (bis |x| > 2 oder Limit)
  return count; // Liegt im Bereich 0..APF_FARBEN
};


void apfel_step(int first, int xsize, int ysize, EA_PIXEL * zielp, \
                float radius)
{
  A_FP ReStep, ImStep, Re0, Im0;
  int x, y, f1, f2, f3;

  A_FP ReStart;
  A_FP ReEnd; // z.B. 2.1 .. 0.5
  A_FP ImStart;
  A_FP ImEnd; // z.B. -1.2 .. 1.2

  static A_FP ReMitte=0.0;
  static A_FP ImMitte=0.0;
  static int YMax;
  static int XMax;
  static EA_PIXEL * ziel;
  static EA_PIXEL apfelpal[APF_FARBEN+1];
  
  if (first == 1)
  { 
#ifdef BIG_APF_PIXELS
    xsize >>= 1;
    ysize >>= 1;
#endif
    XMax = xsize;
    YMax = ysize;
    ziel = zielp;
    if (radius > 0.0) { ReMitte = 0.0; ImMitte = 1.0; 
    }            else { ReMitte = -1.48469562; ImMitte = 0.0; };
    
    if (radius < -41.9) { ReMitte = -0.754571; ImMitte = 0.0; };
       /* (-0.754571,0.0) ist etwa die Einschnuerung in der "Mitte" */
    if (radius >  41.9) { ReMitte = -0.101096365; ImMitte = 0.956286510; };
       /* negative Werte brauchen viele Iterationen bei hohem Zoom  */

    // Bei first == 1 waehlt radius den Mittelpunkt, nicht den Radius!!!
    // 0.0 1.0 war meine Idee, den Rest habe in mit XaoS ausprobiert...
    // dazu auf "r" wie Redraw sitzenbleiben und Statusanzeige lesen :)
    
    for (x=0; x<APF_FARBEN; x++)
    { round2int_inline(y, 255.0 * x / APF_FARBEN);
      apfelpal[x] = RGB( y , y , 255 - y ); // Verlauf von Blau nach Gelb
    };
    apfelpal[APF_FARBEN] = RGB( 84 , 42 , 42 );
    // Farbe der Punkte in der Mandelbrotmenge selbst
  }; // Ende Initialisierung (Ausschnitt, Zoomfaktor, Palette)
  
  if (first == -1)
  { // am Ende Bild so lassen, also nicht nochmal zeichnen
    // sondern gleich zurueckkehren...
    return;
  };

  if ((fabs(radius) / (1.0 + (A_FP)maximum(XMax,YMax))) <= MIN_A_FP)
  { radius = MIN_A_FP * 2.0 * (1.0 + (A_FP)maximum(XMax,YMax)); };

  ReStart  = ReEnd = ReMitte;
  ImStart  = ImEnd = ImMitte;
  ReStart -= radius;
  ReEnd   += radius;
  ImStart -= radius;
  ImEnd   += radius;
  ReStep   = (radius + radius) / (1.0 + (A_FP)XMax);
  ImStep   = (radius + radius) / (1.0 + (A_FP)YMax);

Im0 = ImStart;
ImStep *= 2.0;
for (y=0; y<YMax; y+=2) // zuerst jede 2. Zeile ...
{ Re0 = ReStart;
  f1 = apfelpal[punkt_finden(Re0,Im0)];
  Re0 += ReStep;
  GFXw( 0 , y , ziel ) = f1;
  for (x=1; x<(XMax-1); x+=2)
  { // Iteration ab Re0, Im0 -> Farbe
    f3 = apfelpal[punkt_finden(Re0+ReStep,Im0)];
    if (f1 != f3) { f2 = apfelpal[punkt_finden(Re0,Im0)]; } 
    else          { f2 = f1; };
    // opfert 1 Pixel breite Details fuer Geschwindigkeit
    Re0 += ReStep + ReStep;
    GFXw( x   , y , ziel ) = f2;
    GFXw( x+1 , y , ziel ) = f3;
    f1 = f3;
  };
  Im0 += ImStep;
};

Im0 = ImStart + (0.5 * ImStep);
for (y=1; y<(YMax-1); y+=2) // jetzt jede andere 2. Zeile
{ Re0 = ReStart;
  for (x=0; x<XMax; x++)
  { // Beginn Iteration ab Re0, Im0 -> Farbe
    f1 = GFXr( x , y-1 , ziel );
    if ( f1 != GFXr( x , y+1 , ziel )) 
    { f1 = apfelpal[punkt_finden(Re0,Im0)]; };
    // opfert 1 Pixel hohe Details fuer Geschwindigkeit
    GFXw( x , y , ziel ) = f1;
    Re0 += ReStep;
  };
  Im0 += ImStep;
};

 return;
};
