#include "interferenz.h"

/* Urspruenglich mal ein Pascal-Programm von Inge (1998) */
#define I_FP double

#ifndef INTER_BIGPIX
#define PIX GFX
#else
#define PIX(a,b,c) GFX(a+a,b+b,c) = GFX(a+a+1,b+b,c) = \
                   GFX(a+a+1,b+b+1,c) = GFX(a+a,b+b+1,c)
#endif

void inter_step(int first, int xsize, int ysize, EA_PIXEL * zielp, float tx)
{
  static int xsqrtab[4000];
  static I_FP * sqr2Dtab=NULL;
  static int xmax;
  static int ymax;
  static EA_PIXEL * ziel;
  static I_FP phas1;
  static I_FP phas2;
  static I_FP phas1b;
  static I_FP bonus_r;
  static I_FP lamb1;
  static I_FP lamb2;
  I_FP * tabp;
  int x,y,ysqr;

  if (first==1)
  { 
#ifndef INTER_BIGPIX
    xmax = xsize - 1;
    ymax = ysize - 1;
#else
    xmax = (xsize-1)>>1;
    ymax = (ysize-1)>>1;
#endif
    ziel = zielp;
    phas1 = 0.0;
    phas2 = 0.0;
    phas1b = 0.0;
    // spaeter fuer Anwendung ohne Tabelle extra Abzweigung anlegen
    // lohnt sich bei so einer kleinen Tabelle kaum...
    if (xmax >= 1999)
    { fprintf(stderr,"Interferenz - zuwenig Speicher vorbereitet.\n");
      return;
    };
    sqr2Dtab = (I_FP *)malloc(xmax * ymax * sizeof(I_FP));
    if (sqr2Dtab)
    { tabp = sqr2Dtab;
      for (y=0; y<ymax; y++)
      { ysqr = y * y;
        for (x=0; x<xmax; x++)
        {
          *tabp = sqrt((I_FP)(ysqr + (x*x)));
          tabp++;
        };
      };
    };
    // Wenn mehr Speicher da ist: 2d-Lookuptable fuer sqrt(x^2 + y^2)
    // vom Aussmass xmax * ymax (double) -- Rest durch abs() unnoetig...
    //
    for (x=0; x<xmax; x++) { xsqrtab[2000+x] = xsqrtab[2000-x] = x*x; };
    bonus_r = minimum(0.16*xmax , 0.25*ymax);
    lamb1 = (xmax + ymax) * 0.123;
    lamb2 = lamb1 * 77.0 / 42.0;
  };
  
  if (first==-1)
  {
    free(sqr2Dtab);
    // Bild nicht loeschen
    return;
  };
  
  if (first==0)
  { 
    int e1x = round2int((0.16 * xmax) \
      + (bonus_r * sin(PI2*phas1b)) );
    int e1y = round2int((0.42 * ymax) \
      + (bonus_r * cos(PI2*phas1b)) );
    int e2x = round2int(0.75 * xmax);
    int e2y = round2int(0.75 * ymax);
    I_FP lamb1inv = PI2 / lamb1;
    I_FP lamb2inv = PI2 / lamb2;
    I_FP d1, d2;
    I_FP d1y, d2y;
    int d1x, d2x, d1yi, d2yi;
    int w;
    if (sqr2Dtab)               // dann schnell mit grosser Tabelle vorgehen
    { d1yi = -e1y * xmax; 
      d2yi = -e2y * xmax;
      for (y=0; y<ymax; y++)
      { d1x = -e1x; d2x = -e2x;
        for (x=0; x<xmax; x++)                              //
        { w = round2int(63.0 * (2.0 +                       //
            sin(lamb1inv * sqr2Dtab[abs(d1x)+abs(d1yi)]) +  // INNER LOOP
            sin(lamb2inv * sqr2Dtab[abs(d2x)+abs(d2yi)]))); //
          PIX( x , y , ziel ) = RGB ( w , w , w );          //
          d1x++; d2x++;                                     //
        };
        d1yi += xmax;
        d2yi += xmax;
      };
    }
    else
    { for (y=0; y<ymax; y++)    // langsame Alternative
      { d1x = -e1x; d2x = -e2x;
        d1y = (I_FP)(y - e1y); d1y *= d1y;
        d2y = (I_FP)(y - e2y); d2y *= d2y;
        for (x=0; x<xmax; x++)                               //
        { d1 = lamb1inv * sqrt(d1y + xsqrtab[2000 + d1x]);   //
          d2 = lamb2inv * sqrt(d2y + xsqrtab[2000 + d2x]);   // INNER LOOP
          w = round2int(63.0 * (2.0 + sin(d1) + sin(d2)));   //
          PIX( x , y , ziel ) = RGB ( w , w , w );           //
          // eleganter Grauverlauf...
          d1x++; d2x++;                                      //
        };
      }; // Ende Outer Loop
    };   // Ende langsame Alternative
    
    phas1 -= tx * (lamb1 / 12.3);
    phas2 -= tx * (lamb2 / 18.2);
    phas1b += tx * (1.0 / 54.0);
    if (tx>0.1) { lamb2 *= 1.0 / 1.015; }
       else     { lamb2 *= 1.015 / 1.0; };
  };
  
  return;
};
